@@ -8,9 +8,8 @@ updateCorrPage <- function() {
 }
 
 .hs_on_corr_ccfplot_button_clicked <- function(button) {
-	StateEnv$win$setSensitive(F)
-	on.exit(StateEnv$win$setSensitive(T))
-	setStatusBar("")
+	freezeGUI(use.core.log=F)
+	on.exit(thawGUI())
 	
 	selNames <- iconViewGetSelectedNames(theWidget("selection_iconview"))
 	if (length(selNames) != 2) {
@@ -107,9 +106,8 @@ updateCorrPage <- function() {
 }
 
 .hs_on_corr_relationplot_button_clicked <- function(button) {
-	StateEnv$win$setSensitive(F)
-	on.exit(StateEnv$win$setSensitive(T))
-	setStatusBar("")
+	freezeGUI(use.core.log=F)
+	on.exit(thawGUI())
 	
 	selNames <- iconViewGetSelectedNames(theWidget("selection_iconview"))
 	if (length(selNames) != 2) {
@@ -157,7 +155,7 @@ updateCorrPage <- function() {
 	if (doAggr1 || doAggr2) {
 		aggrBy <- if (doAggr1) { aggr1By } else { aggr2By }
 		aggr.call <- bquote(
-			tmp.data <- lapply(tmp.data, aggregate.timeblob, by=.(aggrBy))
+			tmp.data <- lapply(tmp.data, aggregate.timeblob, by=.(aggrBy), fun.qual="omit")
 		)
 		if (any(grep("( month|year)", aggrBy))) {
 			aggr.call[[3]]$start.month <- hsp$startMonth
@@ -274,9 +272,8 @@ updateCorrPage <- function() {
 
 
 .hs_on_corr_calculate_contiguous_button_clicked <- function(button) {
-	StateEnv$win$setSensitive(F)
-	on.exit(StateEnv$win$setSensitive(T))
-	setStatusBar("")
+	freezeGUI()
+	on.exit(thawGUI())
 	
 	TXV <- theWidget("corr_contiguous_textview")
 	setTextview(TXV, "")
